<!-- 
Transforms a CSPro DCF file from Generic CSPro XML to DDI XML

Authors: Joshua Handley (josh@teleyah.com), Pascal Heus (pheus@opendatafoundation.org)
Version: 2009.02

    Copyright (C) 2006-2009 Open Data Foundation (http://www.opendatafoundation.org)
  
    This program is free software; you can redistribute it and/or modify it under the terms of the
    GNU Lesser General Public License as published by the Free Software Foundation; either version
    2.1 of the License, or (at your option) any later version.
  
    This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
    without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU Lesser General Public License for more details.
  
    The full text of the license is available at http://www.gnu.org/copyleft/lesser.html
-->
<xsl:stylesheet xmlns="http://www.icpsr.umich.edu/DDI" version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" exclude-result-prefixes="xsi">
  <xsl:output method="xml" version="1.0" encoding="UTF-8" cdata-section-elements="qstnLit ivuInstr" indent="yes"/>
  <!-- p_level
		 The name of the Data Dictionary "Level" this DDI is for. 
		 If not specified, the first Level is used.
	-->
  <xsl:param name="p_level"/>
  <!-- p_fileStructure 
		 The file structure can be "hierarchical" (default), "rectangulat" or "relational"
		 - hierarchical: this is the default CSPro format with the data in a single file with records on different lines and identified by a record number
		 - rectangular: all the data is placed in a single file (nto yet supported)
		      NOTE: RECTANGULAR IS NOT CURRENTLY SUPPORTED
		 - relational: multiple data files (one for each record type)

	-->
  <xsl:param name="p_fileStructure">relational</xsl:param>
  <!-- p_filename
		 The name of the output file (if not specified, will use the dictionnary name).
		 This parameter is not used for relational file structure.
	-->
  <xsl:param name="p_filename"/>
  <!-- p_fileExtension 
		 The extension to use for the filename (default is "dat").
	-->
  <xsl:param name="p_fileExtension">dat</xsl:param>
  <!-- p_fileIDPrefix
		 The prefix put in front of the @Name to generate a unique ID
	-->
  <xsl:param name="p_fileIDPrefix">FILE_</xsl:param>
  <!-- p_questionLang
		 Language to choose for question text and instructions 
     (3 char code from CSPro)
	-->
  <xsl:param name="p_questionLang">ENG</xsl:param>
  <!-- p_questionFormat
		 Format to use for question literal and interview instructions.
     Either plain, xhtml or rtf.
	-->
  <xsl:param name="p_questionFormat">plain</xsl:param>
  <!-- START TEMPLATE -->
  <xsl:template match="/">
    <xsl:apply-templates select="/CSProDcf/Dictionary"/>
  </xsl:template>
  <!-- END TEMPLATE -->
  <!-- START [DICTIONARY] TEMPLATE / DDI codeBook -->
  <xsl:template match="Dictionary">
    <!-- DDI codeBook ID = DCF Dictionary Name -->
    <codeBook version="2.1" ID="{/CSProDcf/Dictionary/@Name}" xmlns="http://www.icpsr.umich.edu/DDI" xsi:schemaLocation="http://www.icpsr.umich.edu/DDI http://www.icpsr.umich.edu/DDI/Version2.1.xsd">
      <!-- START DDI DOCUMENT DESCRIPTION (1.0) -->
      <docDscr>
        <citation>
          <titlStmt>
            <titl>
              <!-- DDI Document Title = DCF Dictionary label -->
              <xsl:value-of select="/CSProDcf/Dictionary/Label"/>
            </titl>
          </titlStmt>
          <prodStmt>
            <!-- NOTE: Date function not available in generic XSLT transform -->
            <!-- For Document Production Software, use Version attribute from Dictionary -->
            <software version="2009.03">CSProXML dcf2ddi</software>
          </prodStmt>
        </citation>
      </docDscr>
      <!-- END DDI DOCUMENT DESCRIPTION -->
      <!-- START DDI STUDY DESCRIPTION (2.0)-->
      <stdyDscr>
        <citation>
          <titlStmt>
            <titl>
              <!-- DDI Study Title = DCF Dictionary label -->
              <xsl:value-of select="/CSProDcf/Dictionary/Label"/>
            </titl>
          </titlStmt>
          <prodStmt>
            <!-- For Study Production Software, use the Version attribute from Dictionary -->
            <software version="{/CSProDcf/Dictionary/@Version}">
              <xsl:value-of select="/CSProDcf/Dictionary/@Version"/>
            </software>
          </prodStmt>
        </citation>
      </stdyDscr>
      <!-- END DDI DOCUMENT STUDY -->
      <!-- FILE DESCRIPTION -->
      <xsl:choose>
        <xsl:when test="normalize-space($p_level)">
          <!-- Use specific Level -->
          <xsl:choose>
            <xsl:when test="/CSProDcf/Dictionary/Level[@Name=$p_level]">
              <xsl:apply-templates select="/CSProDcf/Dictionary/Level[@Name=$p_level]"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:comment>
                *** LEVEL <xsl:value-of select="$p_level"/> NOT FOUND ***
              </xsl:comment>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:when>
        <xsl:otherwise>
          <!-- Use first Level -->
          <xsl:apply-templates select="/CSProDcf/Dictionary/Level[1]"/>
        </xsl:otherwise>
      </xsl:choose>
    </codeBook>
  </xsl:template>
  <!-- END [DICTIONARY] TEMPLATE -->
  <!-- [LEVEL] -->
  <xsl:template match="Level">
    <!-- FILE DESCRIPTION -->
    <xsl:choose>
      <xsl:when test="$p_fileStructure='relational'">
        <xsl:apply-templates select="." mode="relational"/>
      </xsl:when>
      <xsl:when test="$p_fileStructure='rectangular'">
        <xsl:apply-templates select="." mode="rectangular"/>
      </xsl:when>
      <xsl:when test="$p_fileStructure='hierarchical'">
        <xsl:apply-templates select="." mode="hierarchical"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:comment>*** UNKNOWN FILE STRUCTURE ***</xsl:comment>
      </xsl:otherwise>
    </xsl:choose>
    <!-- VARIABLE DESCRIPTION -->
    <dataDscr>
      <!--  Variable Groups 4.1  -->
      <xsl:apply-templates select="." mode="varGrp"/>
      <!--  Variable 4.3  (all Item in the Level)	-->
      <!-- Exclude non Item types -->
      <xsl:apply-templates select=".//Item[@ItemType='Item']"/>
    </dataDscr>
  </xsl:template>
  <!-- [LEVEL] RELATIONAL FILE STRUCTURE -->
  <xsl:template match="Level" mode="relational">
    <!-- Each Record in the Level represents a file  -->
    <xsl:for-each select="//Record">
      <xsl:variable name="fileID" select="concat($p_fileIDPrefix,@Name)"/>
      <xsl:variable name="filename">
        <!-- DDI FileName based on Record Name -->
        <xsl:choose>
          <xsl:when test="normalize-space($p_fileExtension)">
            <xsl:value-of select="concat(@Name,'.',$p_fileExtension)"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="@Name"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:variable>
      <fileDscr ID="{$fileID}">
        <fileTxt>
          <!-- DDI File ID = Record Name -->
          <fileName>
            <xsl:value-of select="$filename"/>
          </fileName>
          <fileCont>
            <xsl:value-of select="Label"/>
          </fileCont>
          <fileStrc type="relational">
          </fileStrc>
          <fileType>
            <xsl:value-of select="/CSProDcf/Dictionary/@Version"/>
          </fileType>
        </fileTxt>
      </fileDscr>
    </xsl:for-each>
  </xsl:template>
  <!-- [LEVEL] RECTANGULAR FILE STRUCTURE -->
  <xsl:template match="Level" mode="rectangular">
    <xsl:comment>*** RECTANGULAR FORMAT NOT SUPPORTED ***</xsl:comment>
  </xsl:template>
  <!-- [LEVEL] HIERARCHICAL FILE STRUCTURE -->
  <xsl:template match="Level" mode="hierarchical">
    <!-- HIERARCHICAL FILE STRUCTURE -->
    <!-- The Level is the whole file -->
    <xsl:variable name="fileID" select="concat($p_fileIDPrefix,@Name)"/>
    <xsl:variable name="filename">
      <!-- DDI FileName based on provided $p_filename or Level Name -->
      <xsl:choose>
        <xsl:when test="normalize-space($p_filename)">
          <xsl:value-of select="$p_filename"/>
        </xsl:when>
        <xsl:when test="normalize-space($p_fileExtension)">
          <xsl:value-of select="concat(@Name,'.',$p_fileExtension)"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="@Name"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <fileDscr ID="{concat($p_fileIDPrefix,@Name)}">
      <fileTxt>
        <!-- DDI File ID = DCF Dictionary Name -->
        <fileName>
          <!-- DDI FileName = DCF Dictionary Name -->
          <xsl:value-of select="@Name"/>
        </fileName>
        <fileCont>
          <xsl:value-of select="Label"/>
        </fileCont>
        <fileStrc type="hierarchical">
        </fileStrc>
        <fileType>
          <xsl:value-of select="/CSProDcf/Dictionary/@Version"/>
        </fileType>
      </fileTxt>
    </fileDscr>
  </xsl:template>
  <!-- [LEVEL] VARIABLE GROUPS -->
  <xsl:template match="Level" mode="varGrp">
    <!-- Create a variable group for the LEVEL -->
    <xsl:element name="varGrp">
      <xsl:attribute name="ID">
        <xsl:text>LEVEL_</xsl:text>
        <xsl:value-of select="@Name"/>
      </xsl:attribute>
      <xsl:attribute name="name">
        <xsl:value-of select="@Name"/>
      </xsl:attribute>
      <!-- Create list of children -->
      <xsl:attribute name="varGrp">
        <!-- idItems -->
        <xsl:text>LEVEL_</xsl:text>
        <xsl:value-of select="@Name"/>
        <xsl:text>_IDITEMS </xsl:text>
        <!-- records -->
        <xsl:for-each select="Record">
          <xsl:text>RECORD_</xsl:text>
          <xsl:value-of select="@Name"/>
          <xsl:if test="position()!=last()">
            <xsl:text> </xsl:text>
          </xsl:if>
        </xsl:for-each>
        <!-- Level -->
        <xsl:if test="Level">
          <xsl:text> LEVEL_</xsl:text>
          <xsl:value-of select="Level/@Name"/>
        </xsl:if>
      </xsl:attribute>
      <labl>
        <xsl:value-of select="Label"/>
      </labl>
      <xsl:if test="Note">
        <txt>
          <xsl:value-of select="Note"/>
        </txt>
      </xsl:if>
    </xsl:element>
    <!-- Create a variable group for IDITEMS -->
    <xsl:for-each select="IdItems">
      <xsl:element name="varGrp">
        <xsl:attribute name="ID">
          LEVEL_<xsl:value-of select="../@Name"/>_IDITEMS
        </xsl:attribute>
        <xsl:attribute name="name">
          <xsl:value-of select="../@Name"/>_IDITEMS
        </xsl:attribute>
        <!-- Create list of variables in this group -->
        <xsl:attribute name="var">
          <xsl:for-each select="Item">
            <xsl:text>V</xsl:text>
            <xsl:number count="//Item[@ItemType='Item']" from="/" level="any"/>
            <xsl:if test="position()!=last()">
              <xsl:text> </xsl:text>
            </xsl:if>
          </xsl:for-each>
        </xsl:attribute>
        <labl>
          <xsl:value-of select="../Label"/>
          <xsl:text> Identifiers</xsl:text>
        </labl>
        <xsl:if test="Note">
          <txt>
            <xsl:value-of select="Note"/>
          </txt>
        </xsl:if>
      </xsl:element>
    </xsl:for-each>
    <!-- Create a variable group for every RECORD -->
    <xsl:for-each select="Record">
      <xsl:element name="varGrp">
        <xsl:attribute name="ID">
          RECORD_<xsl:value-of select="@Name"/>
        </xsl:attribute>
        <xsl:attribute name="name">
          <xsl:value-of select="@Name"/>
        </xsl:attribute>
        <!-- Create list of variables in this group -->
        <xsl:attribute name="var">
          <xsl:for-each select="Item">
            <xsl:text>V</xsl:text>
            <xsl:number count="//Item[@ItemType='Item']" from="/" level="any"/>
            <xsl:if test="position()!=last()">
              <xsl:text> </xsl:text>
            </xsl:if>
          </xsl:for-each>
        </xsl:attribute>
        <labl>
          <xsl:value-of select="Label"/>
        </labl>
        <xsl:if test="Note">
          <txt>
            <xsl:value-of select="Note"/>
          </txt>
        </xsl:if>
      </xsl:element>
    </xsl:for-each>
    <!-- Create a variable group sub LEVELS -->
    <xsl:apply-templates select="Level" mode="varGrp"/>
  </xsl:template>
  <xsl:template name="AdjustStartPos">
    <xsl:param name="OrigStart"/>
    <xsl:param name="RecordTypeStart"/>
    <xsl:param name="RecordTypeLen"/>
    <xsl:choose>
      <xsl:when test="$RecordTypeStart &lt; $OrigStart">
        <xsl:value-of select="$OrigStart - $RecordTypeLen"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$OrigStart"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  <!-- START [ITEM] TEMPLATE / DDI Variable (4.3)-->
  <xsl:template match="Item">
    <!-- compute list of var/@Files identifiers-->
    <xsl:variable name="filesID">
      <xsl:choose>
        <xsl:when test="$p_fileStructure='relational'">
          <!-- RELATIONAL -->
          <xsl:choose>
            <xsl:when test="local-name(..)='IdItems'">
              <!-- if this is an ID variable, it will be in every Record in the Level and below -->
              <xsl:for-each select="../..//Record">
                <xsl:if test="position()>1">
                  <xsl:text> </xsl:text>
                </xsl:if>
                <xsl:value-of select="concat($p_fileIDPrefix,@Name)"/>
              </xsl:for-each>
            </xsl:when>
            <xsl:otherwise>
              <!-- this variable is in a single file -->
              <xsl:value-of select="concat($p_fileIDPrefix,../@Name)"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:when>
        <xsl:when test="$p_fileStructure='rectangular'">
          <!-- RECTANGULAR -->
          <!-- TODO -->
        </xsl:when>
        <xsl:otherwise>
          <!-- HIERARCHICAL -->
          <xsl:choose>
            <xsl:when test="normalize-space($p_level)">
              <xsl:value-of select="concat($p_fileIDPrefix,$p_level)"/>
            </xsl:when>
            <xsl:otherwise>
              <!-- Use first Level -->
              <xsl:value-of select="concat($p_fileIDPrefix,/CSProDcf/Dictionary/Level[1]/@Name)"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <var ID="{concat('V',position()) }" name="{@Name}" files="{$filesID}">
      <!-- Decimals -->
      <xsl:if test="@DataType='Numeric' and @Decimal">
        <xsl:attribute name="dcml">
          <xsl:value-of select="@Decimal"/>
        </xsl:attribute>
      </xsl:if>
      <!-- Weights -->
      <xsl:if test="@IsWeight='Yes'">
        <xsl:attribute name="wgt">wgt</xsl:attribute>
      </xsl:if>
      <xsl:if test="@Weight">
        <xsl:attribute name="wgt-var">
          <!-- wgt-var is an idref but Weight is name of variable so we
               we need to compute id from name which is "VXX" where XX
               is the position of the variable in the list of items 
               (not including subitems) -->
          <xsl:variable name="weightvarname" select="@Weight"/>
          <xsl:variable name="weightvarpos" select="count(//Item[@Name=$weightvarname]/preceding::Item[@ItemType='Item']) + 1"/>
          <xsl:value-of select="concat('V',$weightvarpos)"/>
        </xsl:attribute>
      </xsl:if>
      <!-- Location -->
      <xsl:choose>
        <xsl:when test="$p_fileStructure='relational'">
          <!-- RELATIONAL -->
          <!-- In relational files, the record type is removed. The location must therefore be adjsuted accordingly -->
          <xsl:variable name="recordTypeLen" select="/CSProDcf/Dictionary/@RecordTypeLen"/>
          <xsl:variable name="recordTypeStart" select="/CSProDcf/Dictionary/@RecordTypeStart"/>
          <xsl:choose>
            <xsl:when test="local-name(..)='IdItems'">
              <!-- if this is an ID variable, it will be in every Record in the Level and below -->
              <xsl:variable name="item" select="."/>
              <!-- For relational structure we remove record type so we need to adjust start pos
                   appropriately -->
              <xsl:variable name="startPosAdjusted">
                <xsl:call-template name="AdjustStartPos">
                  <xsl:with-param name="OrigStart" select="$item/@Start"/>
                  <xsl:with-param name="RecordTypeStart" select="$recordTypeStart"/>
                  <xsl:with-param name="RecordTypeLen" select="$recordTypeLen"/>
                </xsl:call-template>
              </xsl:variable>
              <xsl:for-each select="../..//Record">
                <location StartPos="{$startPosAdjusted}" EndPos="{$startPosAdjusted + $item/@Len - 1}" width="{$item/@Len}" fileid="{concat($p_fileIDPrefix,@Name)}"/>
              </xsl:for-each>
            </xsl:when>
            <xsl:otherwise>
              <!-- this variable is in a single file -->
              <xsl:variable name="startPosAdjusted">
                <xsl:call-template name="AdjustStartPos">
                  <xsl:with-param name="OrigStart" select="@Start"/>
                  <xsl:with-param name="RecordTypeStart" select="$recordTypeStart"/>
                  <xsl:with-param name="RecordTypeLen" select="$recordTypeLen"/>
                </xsl:call-template>
              </xsl:variable>
              <location StartPos="{$startPosAdjusted}" EndPos="{$startPosAdjusted + @Len - 1}" width="{@Len}" fileid="{concat($p_fileIDPrefix,../@Name)}"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:when>
        <xsl:when test="$p_fileStructure='rectangular'">
          <!-- RECTANGULAR -->
          <!-- TODO -->
        </xsl:when>
        <xsl:otherwise>
          <!-- HIERARCHICAL -->
          <xsl:variable name="fileID" select="$filesID"/>
          <!-- since there is only one file, this is same as the filesID -->
          <xsl:choose>
            <xsl:when test="local-name(..)='IdItems'">
              <!-- if this is an ID variable, it will be in every segment -->
              <xsl:variable name="item" select="."/>
              <xsl:for-each select="../..//Record">
                <location RecSegNo="{@RecordTypeValue}" StartPos="{$item/@Start}" EndPos="{$item/@Start + $item/@Len - 1}" width="{$item/@Len}" fileid="{$fileID}"/>
              </xsl:for-each>
            </xsl:when>
            <xsl:otherwise>
              <!-- this variable is in a single file -->
              <location RecSegNo="{../@RecordTypeValue}" StartPos="{@Start}" EndPos="{@Start + @Len - 1}" width="{@Len}" fileid="{$fileID}"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:otherwise>
      </xsl:choose>
      <!-- Label -->
      <labl>
        <xsl:value-of select="Label"/>
      </labl>
      <!-- QUESTION / DDI qstn (4.3.8) -->
      <xsl:apply-templates select="Question"/>
      <!-- Summary Statistics -->
      <xsl:apply-templates select="SummaryStatistic"/>
      <!-- VALUESET / DDI Category Group (4.3.17) -->
      <xsl:apply-templates select="ValueSet"/>
      <!-- VALUE / DDI Category (4.3.18) -->
      <xsl:apply-templates select="ValueSet/Value"/>
      <!-- VAR FORMAT -->
      <xsl:choose>
        <xsl:when test="@DataType='Numeric'">
          <varFormat type="numeric" schema="other"/>
        </xsl:when>
        <xsl:otherwise>
          <varFormat type="character" schema="other"/>
        </xsl:otherwise>
      </xsl:choose>
      <!-- NOTES -->
      <xsl:if test="Note">
        <notes>
          <xsl:value-of select="Note"/>
        </notes>
      </xsl:if>
      <!-- FORMATTED QUESTION TEXT IN SPECIAL NOTES -->
      <xsl:apply-templates select="Question" mode="notes"/>
    </var>
  </xsl:template>
  <!-- END ITEM TEMPLATE -->
  <!-- START SummaryStatistic TEMPLATE / DDI sumStat (4.3.14) -->
  <xsl:template match="SummaryStatistic">
    <xsl:if test="@Type='Mean' or @Type='Median' or @Type='Min' or @Type='Max' or @Type='Mode' or @Type='StandardDeviation' or @Type='Valid' or @Type='Invalid'">
      <xsl:element name="sumStat">
        <xsl:choose>
          <xsl:when test="@Type='Mean'">
            <xsl:attribute name="type">mean</xsl:attribute>
          </xsl:when>
          <xsl:when test="@Type='Median'">
            <xsl:attribute name="type">medn</xsl:attribute>
          </xsl:when>
          <xsl:when test="@Type='Max'">
            <xsl:attribute name="type">max</xsl:attribute>
          </xsl:when>
          <xsl:when test="@Type='Min'">
            <xsl:attribute name="type">min</xsl:attribute>
          </xsl:when>
          <xsl:when test="@Type='Mode'">
            <xsl:attribute name="type">mode</xsl:attribute>
          </xsl:when>
          <xsl:when test="@Type='StandardDeviation'">
            <xsl:attribute name="type">stdev</xsl:attribute>
          </xsl:when>
          <xsl:when test="@Type='Valid'">
            <xsl:attribute name="type">vald</xsl:attribute>
          </xsl:when>
          <xsl:when test="@Type='Invalid'">
            <xsl:attribute name="type">invd</xsl:attribute>
          </xsl:when>
        </xsl:choose>
        <xsl:if test="@Weighted='Yes'">
          <xsl:attribute name="wgtd">wgtd</xsl:attribute>
        </xsl:if>
        <xsl:value-of select="."/>
      </xsl:element>
    </xsl:if>
  </xsl:template>
  <!-- START [QUESTION] TEMPLATE / DDI qstn (4.3.8) -->
  <xsl:template match="Question">
    <qstn>
      <xsl:if test="QuestionText[@Format=$p_questionFormat and $p_questionLang=@Language]">
        <xsl:element name="qstnLit">
          <xsl:for-each select="QuestionText[@Format=$p_questionFormat and $p_questionLang=@Language]">
            <xsl:apply-templates select=".">
              <xsl:with-param name="format" select="$p_questionFormat"/>
            </xsl:apply-templates>
          </xsl:for-each>
        </xsl:element>
      </xsl:if>
      <xsl:if test="QuestionHelp[@Format=$p_questionFormat and $p_questionLang=@Language]">
        <xsl:element name="ivuInstr">
          <xsl:for-each select="QuestionHelp[@Format=$p_questionFormat and $p_questionLang=@Language]">
            <xsl:apply-templates select=".">
              <xsl:with-param name="format" select="$p_questionFormat"/>
            </xsl:apply-templates>
          </xsl:for-each>
        </xsl:element>
      </xsl:if>
    </qstn>
  </xsl:template>
  <!-- END QUESTION TEMPLATE-->
  <!-- START [QUESTION] TEMPLATE FOR NOTES (RTF/XHTML FORMATTED) -->
  <xsl:template match="Question" mode="notes">
    <xsl:if test="QuestionText[@Format='rtf' and $p_questionLang=@Language]">
      <xsl:element name="notes">
        <xsl:attribute name="type">rtf</xsl:attribute>
        <xsl:attribute name="subject">qstnLit</xsl:attribute>
        <!-- This needs to be in CDATA but can't use cdata-section-elements
          for just this notes node and not all notes in document so
          we hack in a CDATA using xsl-text -->
        <xsl:text disable-output-escaping="yes"><![CDATA[<![CDATA[]]></xsl:text>
        <xsl:for-each select="QuestionText[@Format='rtf' and $p_questionLang=@Language]">
          <xsl:apply-templates select=".">
            <xsl:with-param name="format" select="'rtf'"/>
          </xsl:apply-templates>
        </xsl:for-each>
        <!-- close hacked CDATA-->
        <xsl:text disable-output-escaping="yes">]]&gt;</xsl:text>
      </xsl:element>
    </xsl:if>
    <xsl:if test="QuestionText[@Format='xhtml' and $p_questionLang=@Language]">
      <xsl:element name="notes">
        <xsl:attribute name="type">xhtml</xsl:attribute>
        <xsl:attribute name="subject">qstnLit</xsl:attribute>
        <xsl:text disable-output-escaping="yes"><![CDATA[<![CDATA[]]></xsl:text>
        <xsl:for-each select="QuestionText[@Format='xhtml' and $p_questionLang=@Language]">
          <xsl:apply-templates select=".">
            <xsl:with-param name="format" select="'xhtml'"/>
          </xsl:apply-templates>
        </xsl:for-each>
        <xsl:text disable-output-escaping="yes">]]&gt;</xsl:text>
      </xsl:element>
    </xsl:if>
    <xsl:if test="QuestionHelp[@Format='rtf' and $p_questionLang=@Language]">
      <xsl:element name="notes">
        <xsl:attribute name="type">rtf</xsl:attribute>
        <xsl:attribute name="subject">ivuInstr</xsl:attribute>
        <xsl:text disable-output-escaping="yes"><![CDATA[<![CDATA[]]></xsl:text>
        <xsl:for-each select="QuestionHelp[@Format='rtf' and $p_questionLang=@Language]">
          <xsl:apply-templates select=".">
            <xsl:with-param name="format" select="'rtf'"/>
          </xsl:apply-templates>
        </xsl:for-each>
        <xsl:text disable-output-escaping="yes">]]&gt;</xsl:text>
      </xsl:element>
    </xsl:if>
    <xsl:if test="QuestionHelp[@Format='xhtml' and $p_questionLang=@Language]">
      <xsl:element name="notes">
        <xsl:attribute name="type">xhtml</xsl:attribute>
        <xsl:attribute name="subject">ivuInstr</xsl:attribute>
        <xsl:text disable-output-escaping="yes"><![CDATA[<![CDATA[]]></xsl:text>
        <xsl:for-each select="QuestionHelp[@Format='xhtml' and $p_questionLang=@Language]">
          <xsl:apply-templates select=".">
            <xsl:with-param name="format" select="'xhtml'"/>
          </xsl:apply-templates>
        </xsl:for-each>
        <xsl:text disable-output-escaping="yes">]]&gt;</xsl:text>
      </xsl:element>
    </xsl:if>
  </xsl:template>
  <!-- END QUESTION TEMPLATE-->
  <!-- START [QUESTIONTEXT] TEMPLATE - write question text in given format (
    (xhtml, plain or rtf) -->
  <xsl:template match="QuestionText">
    <xsl:param name="format"/>
    <xsl:if test="@Condition">
      <xsl:choose>
        <xsl:when test="$format='xhtml'">
          <xsl:text disable-output-escaping="yes">&lt;br/&gt;&lt;b&gt;Condition: &lt;/b&gt;</xsl:text>
          <xsl:value-of disable-output-escaping="yes" select="@Condition"/>
          <xsl:text disable-output-escaping="yes">&lt;br&gt;</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>[Condition: </xsl:text>
          <xsl:value-of disable-output-escaping="yes" select="@Condition"/>
          <xsl:text>]
</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:if>
    <xsl:if test="@Occurrences">
      <xsl:choose>
        <xsl:when test="$format='xhtml'">
          <xsl:text disable-output-escaping="yes">&lt;br/&gt;&lt;b&gt;Occurrences: &lt;/b&gt;</xsl:text>
          <xsl:value-of disable-output-escaping="yes" select="@Occurrences"/>
          <xsl:text disable-output-escaping="yes">&lt;br&gt;</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>[Occurrences: </xsl:text>
          <xsl:value-of disable-output-escaping="yes" select="@Occurrences"/>
          <xsl:text>]
</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:if>
    <xsl:value-of disable-output-escaping="yes" select="."/>
  </xsl:template>
  <!-- END QUESTIONTEXT TEMPLATE -->
  <!-- START [VALUESET] TEMPLATE / DDI Category Group (4.3.17)-->
  <xsl:template match="ValueSet">
    <!-- check if at least one Value is not a From-To -->
    <xsl:if test="count(Value/Range[not(@To)])>0">
      <xsl:element name="catgryGrp">
        <xsl:attribute name="ID">
          <xsl:value-of select="@Name"/>
        </xsl:attribute>
        <!-- match the ID of all the Range elements for the catgryGrp (see 4.3.18 template) -->
        <xsl:attribute name="catgry">
          <xsl:for-each select="Value">
            <!-- add a space separator -->
            <xsl:if test="position()!=1">
              <xsl:text> </xsl:text>
            </xsl:if>
            <xsl:for-each select="Range[not(@To)]">
              <xsl:value-of select="../../@Name"/>_<xsl:number count="Value" from="ValueSet"/>_<xsl:value-of select="position()"/><!-- add a space separator (multiple ranges case) --><xsl:if test="position()!=last()">
                <xsl:text> </xsl:text>
              </xsl:if>
            </xsl:for-each>
          </xsl:for-each>
        </xsl:attribute>
        <labl>
          <xsl:value-of select="Label"/>
        </labl>
        <xsl:if test="Note">
          <txt>
            <xsl:value-of select="Note"/>
          </txt>
        </xsl:if>
      </xsl:element>
    </xsl:if>
  </xsl:template>
  <!-- END VALUESET TEMPLATE -->
  <!-- START VALUE= TEMPLATE / DDI Category (4.3.18) -->
  <xsl:template match="Value">
    <!-- Create a catgry for every <Range> element -->
    <!-- But Ignore From-To ranges -->
    <xsl:for-each select="Range[not(@To)]">
      <xsl:element name="catgry">
        <!-- To generate the catgry ID, use the ValueSet Name, the Value position within the ValueSet and the Range position -->
        <xsl:attribute name="ID">
          <xsl:value-of select="../../@Name"/>_<xsl:number count="Value" from="ValueSet"/>_<xsl:value-of select="position()"/>
        </xsl:attribute>
        <xsl:if test="../@Special">
          <xsl:attribute name="missing">Y</xsl:attribute>
          <xsl:attribute name="missType">
            <xsl:value-of select="../@Special"/>
          </xsl:attribute>
        </xsl:if>
        <catValu>
          <xsl:value-of select="@From"/>
          <xsl:if test="@To">
            :<xsl:value-of select="@To"/>
          </xsl:if>
        </catValu>
        <labl>
          <xsl:value-of select="../Label"/>
        </labl>
        <xsl:for-each select="../Frequency">
          <xsl:element name="catStat">
            <xsl:attribute name="type">freq</xsl:attribute>
            <xsl:if test="@Weighted='Yes'">
              <xsl:attribute name="wgtd">wgtd</xsl:attribute>
            </xsl:if>
            <xsl:value-of select="."/>
          </xsl:element>
        </xsl:for-each>
        <xsl:for-each select="../FrequencyPercent">
          <xsl:element name="catStat">
            <xsl:attribute name="type">percent</xsl:attribute>
            <xsl:if test="@Weighted='Yes'">
              <xsl:attribute name="wgtd">wgtd</xsl:attribute>
            </xsl:if>
            <xsl:value-of select="."/>
          </xsl:element>
        </xsl:for-each>
        <xsl:if test="../Note">
          <txt>
            <xsl:value-of select="../Note"/>
          </txt>
        </xsl:if>
      </xsl:element>
    </xsl:for-each>
  </xsl:template>
  <!-- END VALUE TEMPLATE -->
</xsl:stylesheet>
